from .logger import Logger
from .server import Server
from threading import Timer
from flask import Flask, request, jsonify

class Updog:
    Serverpool: list = []
    me: Server
    ClusterMaster: str = ""
    Master: bool = True
    Services: list = []
    serviceTimer: Timer
    slaveTimer: Timer
    log: Logger
    app = Flask(__name__)

    def __init__(self, Serverpool, Master, Services, Logger, port) -> None:
        self.Serverpool = Serverpool
        self.Master = Master
        self.Services = Services
        self.log = Logger
        if not self.Master:
            self.anounceSlave()
        self.log.dev_log("Updog instance created")

        self.app.route('/updog', methods=['GET', 'POST'])(self.updog)
        self.app.route('/getServerpool', methods=['GET'])(self.get_Serverpool)
        self.app.run(port=port)

    def updog(self):
        if request.method == 'POST':
            if request.json:
               data = request.json
               if data["type"] == "server":
                   server = Server(data["url"], data["port"])
                   self.addServer(server)
               else:
                   self.log.error_log("Invalid request")
                   return "Invalid request"
            else:
                self.log.error_log("Invalid request")
                return "Invalid request"

        elif request.method == 'GET':
            return "Status: OK"
        return "Updog"

    def get_Serverpool(self) -> jsonify:
        return jsonify(Serverpool=[e.serialize() for e in self.Serverpool])

#TODO
    def anounceSlave(self) -> None:
        self.log.info_log("Slave anounced")

#TODO
    def anounceMaster(self) -> None:
        pass

#TODO
    def anounceServerpool(self) -> None:
        pass

    def addServer(self, server: Server) -> None:
        self.Serverpool.append(server)
        self.orderServerpool()
        self.anounceServerpool()

    def removeServer(self, server) -> None:
        self.Serverpool.remove(server)

    def addService(self, service) -> None:
        if not callable(service.run()):
            return

        if service not in self.Services:
            self.Services.append(service)

    def removeService(self, service) -> None:
        if not callable(service):
            return

        if service in self.Services:
            self.Services.remove(service)

    def setMaster(self, master: bool) -> None:
        self.Master = master

    def getMaster(self) -> bool:
        return self.Master

    def getServerpool(self) -> list:
        return self.Serverpool

    def getServices(self) -> list:
        return self.Services

    def promoteSlave(self) -> None:
        if self.me.getRank() == 1:
            self.Master = True
            self.orderServerpool()
            self.anounceMaster()
        self.log.error_log("Cant promote slave, not next in line")

    def orderServerpool(self) -> None:
        for i, server in enumerate(self.Serverpool):
           server.setRank(i)

    def demoteMaster(self) -> None:
        self.Master = False

    def runServices(self) -> None:
        for service in self.Services:
            self.log.dev_log("Running service: " + service.__str__())
            res = service.run()
            self.log.dev_log("Service: " + service.__str__() + " returned: " + str(res))

    def checkSceduler(self, time: int) -> None:
        self.serviceTimer = Timer(time, self.checkSceduler)
        self.runServices()

    def checkServerpool(self) -> None:
        for server in self.Serverpool:
           self.log.dev_log("Checking server: " + server)
           res = server.checkServer()
           if res == False:
               self.notify("Server down", server.getIP() + " war master= " + server.getMaster())
               self.removeServer(server)
               self.orderServerpool()
               if server.getMaster() == True:
                   self.promoteSlave()
                   server.setMaster(False)

#TODO
    def notify(self, info: str, msg: str) -> None:
        pass

    def slaveSceduler(self, time: int) -> None:
        self.slaveTimer = Timer(time, self.slaveSceduler)
        self.checkServerpool()



import os
Enviroment = os.getenv("ENVIROMENT")

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

class Logger:
    def __init__(self):
        self.Enviroment = os.getenv("ENVIROMENT")

    def dev_log(self, message):
        if self.Enviroment == "dev" or self.Enviroment == "test":
            print(f"{bcolors.OKBLUE} {message} {bcolors.ENDC}")

    def error_log(self, message):
        print(f"{bcolors.FAIL} {message} {bcolors.ENDC}")

    def warning_log(self, message):
        print(f"{bcolors.WARNING} {message} {bcolors.ENDC}")

    def info_log(self, message):
        print(f"{bcolors.OKBLUE} {message} {bcolors.ENDC}")



from Service import Service
from PostgreSQLBuilder import PostgreSQLBuilder
from HTTPGetBuilder import HTTPGetBuilder

def printRes(result):
    print(result)
    return "asd"

if __name__ == "__main__":
    builder = PostgreSQLBuilder()
    builder.connect("user", "password", "host", 5432, "database")
    builder.query("SELECT * FROM table")
    service = Service(builder)
    service.run()

    builder = HTTPGetBuilder(url="http://www.google.com", onlystatus=True)
    service = Service(builder)
    service.setResFunc(printRes)
    print(service.run())
    print("Service ran")




from updog import Updog
from logger import Logger
from PostgreSQLBuilder import PostgreSQLBuilder
from HTTPGetBuilder import HTTPGetBuilder
from Service import Service
from server import Server
import os

os.environ["ENVIROMENT"] = "dev"


log = Logger()

sql = PostgreSQLBuilder()
sql.connect("user", "password", "host.email", 5432, "database")
sql.query("SELECT * FROM table")
service1 = Service(sql)
print(service1.__str__())

port = 1234
server1 = Server("google.com", 80)

updog = Updog([server1], True, [service1], log)



from .builder import PostgreSQLBuilder, HTTPGetBuilder
from typing import TypeVar

T = TypeVar('T', PostgreSQLBuilder, HTTPGetBuilder)

class Service:
    builder: T
    resFunc = None

    def __init__(self, builder: T) -> None:
        self.builder = builder

    def setResFunc(self, resFunc) -> None:
        if not callable(resFunc):
            return
        self.resFunc = resFunc

    def run(self) -> bool:
        result = self.builder.execute()
        if self.resFunc:
            return self.resFunc(result)
        else:
            if result:
                return True
            else:
                return False

    def __str__(self) -> str:
        return f"Service: {self.builder.__str__()}"



import os
import updog
from logger import Logger

# Create a new updog instance
os.environ["ENVIROMENT"] = "test"
log = Logger()
def service1():
    print("Service 1")

def service2():
    print("Service 2")

def service3():
    print("Service 3")

Serverpool = ["server1", "server2", "server3"]
Services = [service1, service2]
test = updog.Updog(Serverpool, True, Services, log)
test.runServices()
test.checkServerpool()

def test_getServerpool():
    assert test.getServerpool() == ["server1", "server2", "server3"]

def test_getServices():
    assert test.getServices() == [service1, service2]

def test_addServer():
    test.addServer("server4")
    assert test.getServerpool() == ["server1", "server2", "server3", "server4"]

def test_removeServer():
    test.removeServer("server4")
    assert test.getServerpool() == ["server1", "server2", "server3"]

def test_addService():
    test.addService(service3)
    assert test.getServices() == [service1, service2, service3]

def test_removeService():
    test.removeService(service1)
    assert test.getServices() == [service2, service3]
    test.removeService(service3)
    assert test.getServices() == [service2]

def test_fail_addService():
    test.addService("service3")
    assert test.getServices() == [service2]

def test_setMaster():
    test.setMaster(False)
    assert test.getMaster() == False

def test_getMaster():
    assert test.getMaster() == False

def test_promoteSlave():
    test.promoteSlave()
    assert test.getMaster() == True

def main():
    test_getServerpool()
    test_getServices()
    test_addServer()
    test_removeServer()
    test_addService()
    test_removeService()
    test_setMaster()
    test_getMaster()
    test_promoteSlave()
    log.dev_log("All tests passed")

if __name__ == "__main__":
    main()



from setuptools import setup

setup(
    name="updog-py",
    version="0.0.2",
    packages=["updog"],
    install_requires=["psycopg2", "requests", "flask"],
    package_data={"updog": ["docs/*"]},
)



from urllib.request import urlopen, Request

class Server:
    ip: str
    port: int = 80
    master: bool = False
    rank: int = 99

    def __init__(self,  ip: str, port: int):
        self.ip = ip
        self.port = port

    def getIP(self) -> str:
        return self.ip

    def setMaster(self, master: bool) -> None:
        self.master = master
        self.rank = 0

    def getMaster(self) -> bool:
        return self.master

    def setRank(self, rank: int) -> None:
        self.rank = rank

    def getRank(self) -> int:
        return self.rank

    def checkServer(self) -> bool:
        try:
            res = urlopen(self.ip + ":" + str(self.port))
            if res.status == 200:
                return True
            return False
        except Exception as e:
            return False

    def serialize(self) -> dict:
        return {
            "ip": self.ip,
            "port": self.port,
            "master": self.master,
            "rank": self.rank
        }
